const { get, find, isEmpty } = require('lodash')
const { services } = require('pubsweet-component-helper-service')

const { sendNotifications } = require('./notifications/notifications')

const TECHNICAL_STEPS = {
  EQS: 'eqs',
  EQA: 'eqa',
}

const setNewStatus = (step, agree) => {
  if (step === TECHNICAL_STEPS.EQS) {
    return agree ? 'submitted' : 'rejected'
  } else if (step === TECHNICAL_STEPS.EQA) {
    return agree ? 'accepted' : 'pendingApproval'
  }
}

module.exports = ({ Collection, Fragment, User }) => async (req, res) => {
  const { collectionId } = req.params
  const { token, agree, step, comments, customId } = req.body

  try {
    const collections = await Collection.all()
    const collection = await Collection.find(collectionId)
    const technicalCheckToken = get(collection, `technicalChecks.token`)

    if (step === TECHNICAL_STEPS.EQS) {
      if (agree && !customId) {
        return res.status(400).json({
          error: 'A manuscript ID is required.',
        })
      }

      if (agree && find(collections, c => c.customId === customId)) {
        return res.status(400).json({
          error: `CustomID already assigned to a manuscript.`,
        })
      }
    }

    if (isEmpty(technicalCheckToken)) {
      return res.status(400).json({
        error: `Manuscript already handled.`,
      })
    }

    if (technicalCheckToken !== token) {
      return res.status(400).json({
        error: `Invalid token.`,
      })
    }

    delete collection.technicalChecks.token
    if (agree) {
      if (step === TECHNICAL_STEPS.EQA) {
        collection.technicalChecks.eqa = true
      }
      if (step === TECHNICAL_STEPS.EQS) {
        collection.customId = customId
      }
    }

    collection.status = setNewStatus(step, agree)
    await collection.save()

    sendNotifications({
      User,
      agree,
      comments,
      Fragment,
      collection,
      baseUrl: services.getBaseUrl(req),
      isEQA: step === TECHNICAL_STEPS.EQA,
    })

    return res.status(200).json(collection)
  } catch (e) {
    const notFoundError = await services.handleNotFoundError(e, 'Collection')
    return res.status(notFoundError.status).json({
      error: notFoundError.message,
    })
  }
}
